@@ -60,16 +60,19 @@ module Linguist
       generated_net_specflow_feature_file? ||
       composer_lock? ||
       cargo_lock? ||
+      flake_lock? ||
       node_modules? ||
       go_vendor? ||
       go_lock? ||
       poetry_lock? ||
+      pdm_lock? ||
       esy_lock? ||
       npm_shrinkwrap_or_package_lock? ||
       terraform_lock? ||
       generated_yarn_plugnplay? ||
       godeps? ||
       generated_by_zephir? ||
+      htmlcov? ||
       minified_files? ||
       has_source_map? ||
       source_map? ||
@@ -103,7 +106,8 @@ module Linguist
       generated_visualstudio6? ||
       generated_haxe? ||
       generated_jooq? ||
-      generated_pascal_tlb?
+      generated_pascal_tlb? ||
+      generated_sorbet_rbi?
     end
 
     # Internal: Is the blob an Xcode file?
@@ -115,7 +119,7 @@ module Linguist
     def xcode_file?
       ['.nib', '.xcworkspacedata', '.xcuserstate'].include?(extname)
     end
-    
+
     # Internal: Is the blob an IntelliJ IDEA project file?
     #
     # JetBrains IDEs generate project files under an `.idea` directory
@@ -319,7 +323,7 @@ module Linguist
       return false unless extname == '.go'
       return false unless lines.count > 1
 
-      return lines.first(40).any? { |l| l.include? "Code generated by" }
+      return lines.first(40).any? { |l| l =~ %r{^// Code generated .*} }
     end
 
     # Internal: Is the blob a protocol buffer file generated by the
@@ -407,6 +411,13 @@ module Linguist
       !!name.match(/poetry\.lock/)
     end
 
+    # Internal: Is the blob a generated pdm.lock?
+    #
+    # Returns true or false.
+    def pdm_lock?
+      !!name.match(/pdm\.lock/)
+    end
+
     # Internal: Is the blob a generated esy lock file?
     #
     # Returns true or false.
@@ -457,6 +468,13 @@ module Linguist
       !!name.match(/Cargo\.lock/)
     end
 
+    # Internal: Is the blob a generated Nix flakes lock file?
+    #
+    # Returns true or false
+    def flake_lock?
+      !!name.match(/(^|\/)flake\.lock$/)
+    end
+
     # Is the blob a VCR Cassette file?
     #
     # Returns true or false
@@ -760,6 +778,35 @@ module Linguist
       !!name.match(/_tlb\.pas$/i)
     end
 
+    # Internal: Is this a Sorbet RBI file generated by Tapioca?
+    #
+    # Tapioca generates non-human-editable .rbi files in several different
+    # ways:
+    #
+    # 1. `tapioca gem` uses reflection to generate generic .rbi for gems.
+    # 2. `tapioca dsl` uses DSL compilers to generate .rbi for modules/classes.
+    # 3. `tapioca annotations` pulls .rbi from remote sources.
+    #
+    # All are marked with similar wording.
+    #
+    # Returns true or false
+    def generated_sorbet_rbi?
+      return false unless extname.downcase == '.rbi'
+      return false unless lines.count >= 5
+      lines[0].match?(/^# typed:/) &&
+      lines[2].include?("DO NOT EDIT MANUALLY") &&
+      lines[4].match?(/^# Please.*run.*`.*tapioca/)
+    end
+
+    # Internal: Is this an HTML coverage report?
+    #
+    # Tools like coverage.py generate HTML reports under an `htmlcov` directory.
+    #
+    # Returns true or false.
+    def htmlcov?
+      !!name.match(/(?:^|\/)htmlcov\//)
+    end
+
     # Internal: Extract a Hash of name/content pairs from an HTML <meta> tag
     def extract_html_meta(match)
       (match.last.sub(/\/\Z/, "").strip.scan(/
